/*
 * dscr.h.S - This file contains descriptor data tables, and macro definitions
 *   by uratan! 2008.9.8
 */
#ifndef __DSCR_H_S__
#define __DSCR_H_S__

/**************************************************************
 * defined macros within this section are shared with C sources
 */
#include "usb.h"

/*
 * select these options
 */
#define DEVELOP  1			/* use invalid PID when UZK */
	/*
	 * 0 ɂƁAAN2131 ̃IWi PID:0x2131 gp܂B
	 */
#define R_W_U    1			/* enable remote wake up bit */
	/*
	 * 0 ɂƁAconfig descriptor ́uremote wake up ł܂v
	 * ̃rbg𖳌ɂ܂B
	 */
#define MOUSE    1			/* enable mouse as HID3 */
	/*
	 * 0 ɂƁAHID3 ̃}EXfoCX폜܂B
	 * periph.c  #define MOUSE Ɠ悤ύXׂłB
	 */

/*
 * MaxPackeSize for each endpoint
 */
#define DSCR_MPSIZE_EP0      64		/* for control pipe */
#define DSCR_MPSIZE_EP123    64		/* for my endpoints */

/* ep1 */
#define DSCR_ADDR_EP1      0x81		/* Endpoint1 addr */
/* ep2 */
#define DSCR_ADDR_EP2      0x84		/* Endpoint2 addr */
#if MOUSE
/* ep3 */
#define DSCR_ADDR_EP3      0x87		/* Endpoint3 addr */
#endif /* MOUSE */

/*
 * USB HID Class Descriptor Types
 */
#define UDESC_HID                 0x21
#define UDESC_REPORT              0x22


/**************************************************************
 * arm-elf-gcc generats this macro automatically when assemble
 */
#ifdef __ASSEMBLER__

/*
 * firmware version
 */
.set bcdDevice, 0xA022

/*
 * ...
 */
.set bcdUSB, 0x0100		/* USB Specification Release Number */
.set bcdHID, 0x0110		/* HID Class Specification compliance */

/*
 * my device IDs
 */
.set idVendor,  0x0547	/* (Anchor Chips Inc.) */
#if	DEVELOP
//.set idProduct, 0xffff	/* (I expect to be none existance one) */
.set idProduct, 0xeeee	/* (I expect to be none existance one) */
#else	/* DEVELOP */
.set idProduct, 0x2131	/* (EZ-USB - AN2131SC) */
#endif	/* DEVELOP */

/*
 * power requirement
 */
.set bMaxPower, (50/2)	/* [2mA/1] (about 30mA measured by tester) */
	/*
	 * ۂɑ肵 30mA ł
	 */

/*
 * misc constants
 */
.set ET_CONTROL, 0		/* Endpoint type: Control */
.set ET_ISO,     1		/* Endpoint type: Isochronous */
.set ET_BULK,    2		/* Endpoint type: Bulk */
.set ET_INT,     3		/* Endpoint type: Interrupt */

/*
 * macro for 2byte constants in descriptor
 */
.macro dbyte x
	.byte ((\x) & 0xff),(((\x)>>8)&0xff)
.endm
	/*
	 * Lq₷邽߂ɁAUSB ̃GfBAł̃[h`
	 * }Nǉ܂B
	 */

/*
 * share some values between C codes
 */
.global bcdDevice, idVendor, idProduct
.global	DeviceDscr, ConfigDscr, StringDscr
.global	HIDDscr1, ReportDscr1, ReportDscr1Len
.global	HIDDscr2, ReportDscr2, ReportDscr2Len
#if	MOUSE
.global	HIDDscr3, ReportDscr3, ReportDscr3Len
#endif	/* MOUSE */


.text
.arm
.section .text, "ax"

/*-----------------------------------------------------------------------------
 * Global Variables
 *-----------------------------------------------------------------------------
 */
DeviceDscr:
	.byte	0x12			/* Descriptor length */
	.byte	UDESC_DEVICE	/* Decriptor type */
	dbyte	bcdUSB			/* Specification Version (BCD) */
	.byte	0x00			/* Device class */
	.byte	0x00			/* Device sub-class */
	.byte	0x00			/* Device sub-sub-class */
	.byte	DSCR_MPSIZE_EP0	/* Maximum packet size */
	dbyte	idVendor		/* Vendor ID */
	dbyte	idProduct		/* Product ID */
	dbyte	bcdDevice		/* Product version ID */
	.byte	1				/* Manufacturer string index */
	.byte	2				/* Product string index */
	.byte	0				/* Serial number string index */
	.byte	1				/* Numder of configurations */

ConfigDscr:
	.byte	0x09			/* Descriptor length */
	.byte	UDESC_CONFIG	/* Descriptor type */
	dbyte	ConfigLength	/* Configuration -> End Points length */
#if	MOUSE
	.byte	3				/* Number of interfaces */
#else	/* MOUSE */
	.byte	2				/* Number of interfaces */
#endif	/* MOUSE */
	.byte	1				/* Configuration number */
	.byte	0				/* Configuration string index */
#if	R_W_U
	.byte	0b10100000		/* Attributes (b7:buspwr, b6:selfpwr, b5:rwu) */
#else	/* R_W_U */
	.byte	0b10000000		/* Attributes (b7:buspwr, b6:selfpwr, b5:rwu) */
#endif	/* R_W_U */
	.byte	bMaxPower		/* Power requirement [x2mA] */


/*
 * first interface is used as KBD, with endpoint 1
 */
IntrfcDscr1:
	.byte	0x09			/* Descriptor length */
	.byte	UDESC_INTERFACE	/* Descriptor type */
	.byte	0				/* Zero-based index of this interface */
	.byte	0				/* Alternate setting */
	.byte	1				/* Number of end points */
	.byte	0x03			/* Interface class (HID) */
	.byte	0x01			/* Interface sub class (boot device) */
	.byte	0x01			/* Interface protocol (keyboard) */
	.byte	3				/* Interface descriptor string index */

HIDDscr1:
	.byte	0x09			/* Descriptor length */
	.byte	UDESC_HID		/* Descriptor type */
	dbyte	bcdHID			/* HID Class Specification compliance */
	.byte	0				/* Country localization (=none) */
	.byte	1				/* Number of descriptors to follow */
	.byte	UDESC_REPORT	/* And it's a Report descriptor */
	dbyte	ReportDscr1Len

EpI1Dscr:
	.byte	0x07			/* Descriptor length */
	.byte	UDESC_ENDPOINT	/* Descriptor type */
	.byte	DSCR_ADDR_EP1	/* Endpoint number, and direction */
	.byte	ET_INT			/* Endpoint type */
	dbyte	DSCR_MPSIZE_EP123	/* Maximun packet size */
	.byte	10				/* Polling interval [msec] */


/*
 * second interface is used as control, with endpoint 2
 */
IntrfcDscr2:
	.byte	0x09			/* Descriptor length */
	.byte	UDESC_INTERFACE	/* Descriptor type */
	.byte	1				/* Zero-based index of this interface */
	.byte	0				/* Alternate setting */
	.byte	1				/* Number of end points */
	.byte	0x03			/* Interface class (HID) */
	.byte	0x00			/* Interface sub class (no subclass) */
	.byte	0x00			/* Interface protocol (none) */
	.byte	4				/* Interface descriptor string index */

HIDDscr2:
	.byte	0x09			/* Descriptor length */
	.byte	UDESC_HID		/* Descriptor type */
	dbyte	bcdHID			/* HID Class Specification compliance */
	.byte	0				/* Country localization (=none) */
	.byte	1				/* Number of descriptors to follow */
	.byte	UDESC_REPORT	/* And it's a Report descriptor */
	dbyte	ReportDscr2Len

EpI2Dscr:
	.byte	0x07			/* Descriptor length */
	.byte	UDESC_ENDPOINT	/* Descriptor type */
	.byte	DSCR_ADDR_EP2	/* Endpoint number, and direction */
	.byte	ET_INT			/* Endpoint type */
	dbyte	DSCR_MPSIZE_EP123	/* Maximun packet size */
	.byte	10				/* Polling interval [msec] */

#if	MOUSE
/*
 * third interface is used as mouse, with endpoint 3
 */
IntrfcDscr3:
	.byte	0x09			/* Descriptor length */
	.byte	UDESC_INTERFACE	/* Descriptor type */
	.byte	2				/* Zero-based index of this interface */
	.byte	0				/* Alternate setting */
	.byte	1				/* Number of end points */
	.byte	0x03			/* Interface class (HID) */
	.byte	0x01			/* Interface sub class (boot device) */
	.byte	0x02			/* Interface protocol (mouse) */
	.byte	5				/* Interface descriptor string index */

HIDDscr3:
	.byte	0x09			/* Descriptor length */
	.byte	UDESC_HID		/* Descriptor type */
	dbyte	bcdHID			/* HID Class Specification compliance */
	.byte	0				/* Country localization (=none) */
	.byte	1				/* Number of descriptors to follow */
	.byte	UDESC_REPORT	/* And it's a Report descriptor */
	dbyte	ReportDscr3Len

EpI3Dscr:
	.byte	0x07			/* Descriptor length */
	.byte	UDESC_ENDPOINT	/* Descriptor type */
	.byte	DSCR_ADDR_EP3	/* Endpoint number, and direction */
	.byte	ET_INT			/* Endpoint type */
	dbyte	DSCR_MPSIZE_EP123	/* Maximun packet size */
	.byte	10				/* Polling interval [msec] */
#endif	/* MOUSE */

/*
 * configration descriptor includes to here
 */
.set ConfigLength, .-ConfigDscr

	.byte	0	/* <=== IMPORTANT: terminator for ucore_SetInterface() */


	/*
	 * ȉAHID foCXƂĂ report descriptor ̋LqłB
	 */
/*
 * Standard USB Keyboard
 *
 *  KBD->host (8bytes)
 *  +-+-+-+-+-+-+-+-+ +----+ +--------+ +-----+ +-----+ +-----+ +-----+ +-----+
 *  |x x x x x x x x| |rsrv| |keycode1| | kc2 | | kc3 | | kc4 | | kc5 | | kc6 |
 *  +-+-+-+-+-+-+-+-+ +----+ +--------+ +-----+ +-----+ +-----+ +-----+ +-----+
 *   | | | | | | | |
 *   | | | | | | | LEFT CTRL
 *   | | | | | | LEFT SHIFT
 *   | | | | | LEFT ALT
 *   | | | | LEFT GUI             NUM LOCK
 *   | | | RIGHT CTRL          CAPS LOCK |
 *   | | RIGHT SHIFT       SCROLL LOCK | |
 *   | RIGHT ALT             COMPOSE | | |
 *   RIGHT GUI                KANA | | | |
 *                               | | | | |
 *  host->KBD             +-+-+-+-+-+-+-+-+
 *     (1 byte for LEDs)  |0 0 0 x x x x x|
 *                        +-+-+-+-+-+-+-+-+
 */
ReportDscr1:
	.byte	0x05, 0x01		/* Usage_Page (Generic Desktop) */
	.byte	0x09, 0x06		/* Usage (Keyboard) */
	.byte	0xA1, 0x01		/* Collection (Application) */
	.byte	0x05, 0x07		/*   Usage page (Key Codes) */
	.byte	0x19, 0xE0		/*   Usage_Minimum (224) */
	.byte	0x29, 0xE7		/*   Usage_Maximum (231) */
	.byte	0x15, 0x00		/*   Logical_Minimum (0) */
	.byte	0x25, 0x01		/*   Logical_Maximum (1) */
	.byte	0x75, 0x01		/*   Report_Size (1) */
	.byte	0x95, 0x08		/*   Report_Count (8) */
	.byte	0x81, 0x02		/*   Input (Data,Var,Abs) = Modifier Byte */
	.byte	0x81, 0x01		/*   Input (Constant) = Reserved Byte */
	.byte	0x19, 0x00		/*   Usage_Minimum (0) */
//	.byte	0x29, 0xe7		/*   Usage_Maximum (231) */
	.byte	0x2a, 0xff, 0x00/*   Usage_Maximum (255) */
	.byte	0x15, 0x00		/*   Logical_Minimum (0) */
//	.byte	0x25, 0xe7		/*   Logical_Maximum (231) */
	.byte	0x26, 0xff, 0x00/*   Logical_Maximum (255) */
	.byte	0x75, 0x08		/*   Report_Size (8) */
	.byte	0x95, 0x06		/*   Report_Count (6) */
	.byte	0x81, 0x00		/*   Input (Data,Array) = Keycode Bytes(6) */
	.byte	0x05, 0x08		/*   Usage Page (LEDs) */
	.byte	0x19, 0x01		/*   Usage_Minimum (1) */
	.byte	0x29, 0x05		/*   Usage_Maximum (5) */
	.byte	0x15, 0x00		/*   Logical_Minimum (0) */
	.byte	0x25, 0x01		/*   Logical_Maximum (1) */
	.byte	0x75, 0x01		/*   Report_Size (1) */
	.byte	0x95, 0x05		/*   Report_Count (5) */
	.byte	0x91, 0x02		/*   Output (Data,Var,Abs) = LEDs (5 bits) */
	.byte	0x95, 0x03		/*   Report_Count (3) */
	.byte	0x91, 0x01		/*   Output (Constant) = Pad (3 bits) */
	.byte	0xC0			/* End_Collection */
.set ReportDscr1Len, .-ReportDscr1

/*
 * vendor specific report
 */
	/*
	 * UZK p̃foCXłB
	 */
ReportDscr2:
	.byte	0x06, 0xA0, 0xFF/* Usage Page (FFA0H = vendor defined) */
	.byte	0x09, 0x01		/* Usage (Vendor defined) */
	.byte	0xA1, 0x01		/* Collection (Application) */
	.byte	0x09, 0x02		/* Usage (vendor defined) */
	.byte	0xA1, 0x00		/* Collection (Physical) */
	.byte	0x06, 0xA1, 0xFF/* Usage Page (vendor defined) */
/* The input report (2 byte) */
	.byte	0x09, 0x03		/* Usage (vendor defined) */
	.byte	0x09, 0x04		/* Usage (vendor defined) */
	.byte	0x15, 0x80		/* Logical minimum (80H = -128) */
	.byte	0x25, 0x7F		/* Logical maximum (7FH = 127) */
	.byte	0x35, 0x00		/* Physical minimum (0) */
	.byte	0x45, 0xFF		/* Physical maximum (255) */
	.byte	0x75, 0x08		/* Report size (8 bits) */
	.byte	0x95, 0x02		/* Report count (1 fields) */
	.byte	0x81, 0x02		/* Input (Data, Variable, Absolute) */
/* The output report (9 bytes) */
	.byte	0x09, 0x05		/* Usage (vendor defined) */
	.byte	0x09, 0x06		/* Usage (vendor defined) */
	.byte	0x15, 0x80		/* Logical minimum (80H = -128) */
	.byte	0x25, 0x7F		/* Logical maximum (7FH = 127) */
	.byte	0x35, 0x00		/* Physical minimum (0) */
	.byte	0x45, 0xFF		/* Physical maximum (255) */
	.byte	0x75, 0x08		/* Report size (8 bits) */
	.byte	0x95, 0x09		/* Report count (9 fields) */
	.byte	0x91, 0x02		/* Output (Data, Variable, Absolute) */
	.byte	0xC0			/* End Collection (Physical) */
	.byte	0xC0			/* End Collection (Application) */
.set ReportDscr2Len, .-ReportDscr2

#if	MOUSE
/*
 * 5 Key Magic Ball Mouse (Arvel)
 *
 *  mouse->host (4bytes)
 *  +-+-+-+-+-+-+-+-+           +--------+ +--------+ +--------+
 *  |          x x x|           |   dX   | |   dY   | | Wheel  |
 *  +-+-+-+-+-+-+-+-+           +--------+ +--------+ +--------+
 *         | | | | |                            -         +1
 *         | | | | LEFT BUTTON                  |          |
 *         | | | RIGHT BUTTON    - <===> +      |     +7 <=|=> -7
 *         | | CENTER BUTTON                    |          |
 *         | LEFT BODY BUTTON                   +          -1
 *         RIGHT BODY BUTTON                         (one of 4 values)
 */
	/*
	 * 莝 Magic Ball Mouse ̗łAقڕWIȂ悤ł
	 */
ReportDscr3:
	.byte	0x05, 0x01		/* Usage_Page (Generic Desktop) */
	.byte	0x09, 0x02		/* Usage (Mouse) */
	.byte	0xA1, 0x01		/* Collection (Application) */
	.byte	0x09, 0x01		/* Usage (Pointer) */
	.byte	0xA1, 0x00		/* Collection (Physical) */
	.byte	0x05, 0x09		/* Usage_Page (Button) */
	.byte	0x19, 0x01		/*   Usage_Minimum (1) */
	.byte	0x29, 0x05		/*   Usage_Maximum (5) */
	.byte	0x15, 0x00		/*   Logical_Minimum (0) */
	.byte	0x25, 0x01		/*   Logical_Maximum (1) */
	.byte	0x75, 0x01		/*   Report_Size (1) */
	.byte	0x95, 0x05		/*   Report_Count (5) */
	.byte	0x81, 0x02		/*   Input (Data,Var,Abs) = Button (5bits) */
	.byte	0x95, 0x03		/*   Report_Count (3) */
	.byte	0x81, 0x01		/*   Input (Constant) = Reserved (3bits) */
	.byte	0x05, 0x01		/* Usage_Page (Generic Desktop) */
	.byte	0x09, 0x30		/* Usage (X) */
	.byte	0x09, 0x31		/* Usage (Y) */
	.byte	0x09, 0x38		/* Usage (Wheel) */
	.byte	0x15, 0x81		/*   Logical_Minimum (-127) */
	.byte	0x25, 0x7f		/*   Logical_Maximum (127) */
	.byte	0x75, 0x08		/*   Report_Size (8) */
	.byte	0x95, 0x03		/*   Report_Count (3) */
	.byte	0x81, 0x06		/*   Input (Data,Var,Rel) = Mouse Motion */
	.byte	0xC0			/* End_Collection */
	.byte	0xC0			/* End_Collection */
.set ReportDscr3Len, .-ReportDscr3
#endif	/* MOUSE */


/*
 * strings
 */
StringDscr:
StringDscr0:
	.byte	StringDscr0Len	/* String descriptor length */
	.byte	UDESC_STRING
	dbyte	0x0409
.set StringDscr0Len, .-StringDscr0

/*
 * 1 for Manufacturer string
 */
StringDscr1:
	.byte	StringDscr1Len	/* String descriptor length */
	.byte	UDESC_STRING
	dbyte	'A'				/* "Anchor Chips Inc." */
	dbyte	'n'
	dbyte	'c'
	dbyte	'h'
	dbyte	'o'
	dbyte	'r'
	dbyte	' '
	dbyte	'C'
	dbyte	'h'
	dbyte	'i'
	dbyte	'p'
	dbyte	's'
	dbyte	' '
	dbyte	'I'
	dbyte	'n'
	dbyte	'c'
	dbyte	'.'
.set StringDscr1Len, .-StringDscr1

	/*
	 * ȉAUZK ֌W string łBR[hZk̂ߒZZĂ܂
	 */
/*
 * 2 for Product string
 */
StringDscr2:
	.byte	StringDscr2Len	/* Descriptor length */
	.byte	UDESC_STRING
	dbyte	'a'				/* "arm-UZK" */
	dbyte	'r'
	dbyte	'm'
	dbyte	'-'
	dbyte	'U'
	dbyte	'Z'
	dbyte	'K'
.set StringDscr2Len, .-StringDscr2

/*
 * 3 for name of keybaord
 */
StringDscr3:
	.byte	StringDscr3Len	/* Descriptor length */
	.byte	UDESC_STRING
	dbyte	'K'				/* "KB" */
	dbyte	'B'
.set StringDscr3Len, .-StringDscr3

/*
 * 4 for name of UZK
 */
StringDscr4:
	.byte	StringDscr4Len	/* Descriptor length */
	.byte	UDESC_STRING
	dbyte	'C'				/* "CTRL" */
	dbyte	'T'
	dbyte	'R'
	dbyte	'L'
.set StringDscr4Len, .-StringDscr4

#if	MOUSE
/*
 * 5 for name of mouse
 */
StringDscr5:
	.byte	StringDscr5Len	/* Descriptor length */
	.byte	UDESC_STRING
	dbyte	'M'				/* "MS" */
	dbyte	'S'
.set StringDscr5Len, .-StringDscr5
#endif	/* MOUSE */


.end
#endif /* __ASSEMBLER__ */

#endif /* __DSCR_H_S__ */
