/*
 * main.c:  main C entry
 *   by uratan! 2008.9.8
 */
#include "common.h"
#include "LPC214x.h"
#include "irq.h"
#include "swi.h"
#include "pcon.h"

/*
 * functions I have
 */
int main(void);
void sys_init(void);
//void IRQ_handler(void)   __attribute__ ((interrupt("IRQ")));
void FIQ_handler(void)   __attribute__ ((interrupt("FIQ")));
//void SWI_handler(void)   __attribute__ ((interrupt("SWI")));
void UNDEF_handler(void) __attribute__ ((interrupt("UNDEF")));
void PABT_handler(void)  __attribute__ ((interrupt("ABORT")));
void DABT_handler(void)  __attribute__ ((interrupt("ABORT")));

/*
 * externs
 */
/* in dscr.S */
extern u_long *bcdDevice, *idVendor, *idProduct;
/* in timer0.c */
extern int timer0_init(void);
extern void timer0_run(void);
extern void timer0_stop(void);
extern volatile int timer0_tick;
/* in uart1.c */
extern int uart1_init(u_long baudrate);
extern int peek_recv(void);
extern void set_to_send(u_char c);
/* in pdown.c */
extern void pdown_init(void);
extern void power_down(void);
extern void pdown_sys_pll_init(void);
/* in usbhw.c */
extern void uhw_Connect(int con);
/* in usbcore.c */
extern int ucore_InitialCore(void);
extern void ucore_SetupCommand(void);
extern u_char SETUPDAT[8];
extern int GotSUD;
/* in periph.c */
extern void TD_Init(void);
void extern TD_Poll(void);

/*
 * variables
 */
int counter = 0;


/*
 * main entry
 */
int main(void)
{
	/*
	 * setup misc system configuration
	 */
	sys_init();
	init_VIC();

	/*
	 * initialize I/O pins
	 */
	SCS |= 0x03;		/* both GPIO are used as Fast I/O */
	PINSEL0 = 0;			/* confirm reset values */
	PINSEL1 = 0;
	FIO0DIR = 0x00000000;
	PINSEL2 = 0;
	FIO1DIR = 0x00000000;

	/*
	 * setup debug UART port
	 */
	cm_init_serial0(57600);			/* setup debug uart port */
	rprintf(
		"\r\n"
		"/*\r\n"
		" * start arm-uzk\r\n"
		" *   bcdDevice: 0x%04x\r\n"
		" * idVendor: 0x%04x  idProduct: 0x%04x\r\n"
		" */\r\n",
		&bcdDevice,
		&idVendor, &idProduct
	);

	/*
	 * to drive 'STAT' LED and debug I/O
	 */
	FIO0DIR |= LED_MON;
	MON_LED_ON;

	FIO1DIR |= LED_STAT | DEBUG_PORT | DEBUG_PORT2;
	FIO1SET = LED_STAT;						/* turn off LED */
	FIO1CLR = DEBUG_PORT | DEBUG_PORT2;		/* set low */

	/*
	 * initial peripherals
	 */
	uart1_init(9600);
	timer0_init();
	pdown_init();
	ucore_InitialCore();	/* initial about USB */
	TD_Init();

	/*
	 * here we go
	 */
	timer0_run();
	IntEnable();
	uhw_Connect(TRUE);		/* USB Connect */

//	set_to_send('1');
//	set_to_send('u');
//	set_to_send('2');
//	set_to_send('u');
//	set_to_send('3');

	/* Power Down test */
//	power_down();

	/*
	 * The main Function is an endless loop
	 */
	while(1) {
//		int k;

		/*
		 * Wait for SUDAV
		 */
		if(GotSUD) {
			/*
			 * we can make the hast wait until feeding the data here,
			 * without host-timeout
			 */
	FIO1SET = DEBUG_PORT;		/* set high */
			ucore_SetupCommand();	/* SUDAV flag is cleared there */
	FIO1CLR = DEBUG_PORT;		/* set low */
		}


//		k = cm_keyin();
//		if(k != -1) {
//			cm_putchar(k);
//		}

		if(timer0_tick) {
			//timer0_tick = FALSE;	/* TD_Poll() shall clear */

			if(++counter >= 250) {
				counter = 0;

//				set_to_send('.');
//				while(1) {
//					k = peek_recv();
//					if(k == -1)
//						break;
//					set_to_send(k);
//				}

				/*
				 * toggle LED every 500msec
				 */
				if(FIO1PIN & LED_STAT) {
					FIO1CLR = LED_STAT;
//					MON_LED_OFF;
				} else {
					FIO1SET = LED_STAT;
//					MON_LED_ON;
				}
			}
		}

		TD_Poll();
	}

	/* never return from main() */
	while(1)
		;
}


/* ===============================================================
 * setup core hardware running conditions
 */
void sys_init(void)
{
	/*
	 * select vector memory
	 */
	MEMMAP = 0x01;		/* User Flash Mode */

	/*
	 * Setting peripheral Clock (PCLK) to System Clock (CCLK)
	 *   0: PCLK = 1 * CCLK / 4
	 *   1: PCLK = 4 * CCLK / 4
	 *   2: PCLK = 2 * CCLK / 4
	 */
#if (Fpclk / (Fcclk / 4)) == 1
	VPBDIV = 0;						/* <== 15MHz */
#elif (Fpclk / (Fcclk / 4)) == 2
	VPBDIV = 2;						/* <== 30MHz */
#elif (Fpclk / (Fcclk / 4)) == 4
	VPBDIV = 1;
#else
#error invalid VPBDIV
#endif

	/*
	 * setup core clock PLL
	 */
	pdown_sys_pll_init();
		/*
		 * 011: 60mA
		 *  -PLL: 30mA (-30mA)
		 *  +PER: 64mA ( +4mA)
		 */

	/*
	 * config memory accelerater module
	 */
	MAMCR = 0;

#if Fcclk < 20000000
	MAMTIM = 1;
#elif Fcclk < 40000000
	MAMTIM = 2;
#else
	MAMTIM = 3;				/* <== */
#endif

	MAMCR = 2;

	/*
	 * save powers
	 */
	PCONP &= ~(PP_PCAD0 | PP_PCAD1);	/* turn off unused peripherals */
	PCONP &= ~(PP_PCI2C0 | PP_PCI2C1);
	PCONP &= ~(PP_PCSPI0 | PP_PCSPI1);
	PCONP &= ~(PP_PCRTC | PP_PCPWM0);
//	PCONP |= PP_PUSB;	/* turn on USB block */
	return;
}

/* ===============================================================
 * exception handlers
 */
void FIQ_handler(void)
{
	rprintf("!!FIQ\r\n");
	while(1)
		;		/* halt at here */
}

void UNDEF_handler(void)
{
	rprintf("!!UNDEF\r\n");
	while(1)
		;		/* halt at here */
}

void PABT_handler(void)
{
	rprintf("!!PABT\r\n");
	while(1)
		;		/* halt at here */
}

void DABT_handler(void)
{
	rprintf("!!DABT\r\n");
	while(1)
		;		/* halt at here */
}
